function ibeta( alpha, p, q, ier )

!-----------------------------------------------------------------------
!     Inverse of the incomplete Beta function
!
!     ALPHA - Input . Probability                             - Real
!     P     - Input . First parameter  (P > 0)                - Real
!     Q     - Input . Second parameter (Q > 0)                - Real
!     IER   - Output. Return code                             - Integer
!                     0 = no error
!                     1 = invalid input argument
!                     2 = error
!
!     External functions called:
!       BETACDF (and DLGAMA if not in Fortran library)
!
!     Fortran functions called:
!       ABS  EXP  LOG  SQRT (and DLGAMA if available)
!
!     Algorithm from algorithm AS 109:
!     GW Cran, KJ Martin, GE Thomas,
!     Remark AS R19 and Algorithm AS 109:
!     A Remark on Algorithms AS 63: The Incomplete Beta Integral
!     and AS 64: Inverse of the Incomplete Beta Integeral,
!     Applied Statistics,
!     Volume 26, Number 1, 1977, pages 111-114.
!
!     FORTRAN90 version by John Burkardt
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   real(kind=8) :: ibeta

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: alpha, p, q
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

!! real(kind=8), external :: dlgama
   real(kind=8), external :: betacdf

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: sae=-37.0D+00

   !     These constants are machine dependent:
   !     sae = the most negative decimal exponent
   !           which does not cause an underflow.

   real(kind=8) :: a, acu, adj, beta, fpu, g, h, pp, prev, qq,  &
                   r, s, sq, t, tx, w, xin, y, yprev
   integer :: iex
   logical :: lindx

   !--------------------------------------------------------------------



   fpu = 10.0D+00**sae

   ier = 0
   ibeta = alpha

   !  Test for admissibility of parameters.

   if ( p <= zero .or. q <= zero ) then
     ier = 1
     return
   end if

   if ( alpha < zero .or. one < alpha ) then
     ier = 1
     return
   end if

   if ( alpha == zero .or. alpha == one ) then
     return
   end if

   beta = dlgama(p) + dlgama(q) - dlgama(p+q)

   !  Change tail if necessary.

   if ( 0.5D+00 < alpha ) then
     a = one - alpha
     pp = q
     qq = p
     lindx = .true.
   else
     a = alpha
     pp = p
     qq = q
     lindx = .false.
   end if

   !  Calculate the initial approximation.

   r = sqrt ( - log ( a * a ) )

   y = r - ( 2.30753D+00 + 0.27061D+00 * r ) &
     / ( one + ( 0.99229D+00 + 0.04481D+00 * r ) * r )

   if ( one < pp .and. one < qq ) then

     r = ( y * y - 3.0D+00 ) / 6.0D+00
     s = one / ( pp + pp - one )
     t = one / ( qq + qq - one )
     h = 2.0D+00 / ( s + t )
     w = y * sqrt ( h + r ) / h - ( t - s ) &
     * ( r + 5.0D+00 / 6.0D+00 - 2.0D+00 / ( 3.0D+00 * h ) )
     ibeta = pp / ( pp + qq * exp ( w + w ) )

   else

     r = qq + qq
     t = one / ( 9.0D+00 * qq )
     t = r * ( one - t + y * sqrt ( t ) )**3

     if ( t <= zero ) then
       ibeta = one - exp ( ( log ( ( one - a ) * qq ) + beta ) / qq )
     else

       t = ( 4.0D+00 * pp + r - 2.0D+00 ) / t

       if ( t <= one ) then
         ibeta = exp ( ( log ( a * pp ) + beta ) / pp )
       else
         ibeta = one - 2.0D+00 / ( t + one )
       end if

     end if

   end if

   !  Solve for X by a modified Newton-Raphson method,
   !  using the function betacdf.

   r = one - pp
   t = one - qq
   yprev = zero
   sq = one
   prev = one

   if ( ibeta < 0.0001D+00 ) then
     ibeta = 0.0001D+00
   end if

   if ( 0.9999D+00 < ibeta ) then
     ibeta = 0.9999D+00
   end if

   iex = max ( - 5.0D+00 / pp**2 - one / a**0.2D+00 - 13.0D+00, sae )

   acu = 10.0D+00**iex

   do

     y = betacdf( ibeta, pp, qq, ier )

     if ( ier /= 0 ) then
       ier = 2
       return
     end if

     xin = ibeta
     y = ( y - a ) * exp ( beta + r * log ( xin ) + t * log ( one - xin ) )

     if ( y * yprev <= zero ) then
       prev = max ( sq, fpu )
     end if

     g = one

     do

       do

         adj = g * y
         sq = adj * adj

         if ( sq < prev ) then

           tx = ibeta - adj

           if ( zero <= tx .and. tx <= one ) then
             exit
           end if

         end if

         g = g / 3.0D+00

       end do

       if ( prev <= acu ) then
         if ( lindx ) then
           ibeta = one - ibeta
         end if
         return
       end if

       if ( y * y <= acu ) then
         if ( lindx ) then
           ibeta = one - ibeta
         end if
         return
       end if

       if ( tx /= zero .and. tx /= one ) then
         exit
       end if

       g = g / 3.0D+00

     end do

     if ( tx == ibeta ) then
       exit
     end if

     ibeta = tx
     yprev = y

   end do

   if ( lindx ) then
     ibeta = one - ibeta
   end if

end function ibeta
