function ibinom( pbin, n, p, ier )

!-----------------------------------------------------------------------
!
!     Inverse of the binomial cdf.
!     Given that a random variable is distributed according to the
!     binomial distribution with sample size N and probability parameter
!     P, it returns the smallest integer value such that the probability
!     that the random variable is less than or equal to that value is
!     at least PBIN.
!
!     PBIN  - Input . Input probability (cdf) (0<= PBIN <=1)  - Real
!     N     - Input . Sample size (N > 0)                     - Integer
!     P     - Input . Probability of success (0<= P <=1)      - Real
!     IER   - Output. Return code :                           - Integer
!                     0 = normal
!                     1 = invalid input argument
!                     3 = error in BINOMCDF
!
!     External functions called:
!       BINOMCDF  BINOMPMF
!
!     Fortran functions called:
!       ABS  LOG  MAX  MIN  SQRT
!
!-----------------------------------------------------------------------

   implicit none

   !  Function
   !  --------

   integer :: ibinom

   !  Arguments
   !  ---------

   real(kind=8), intent(in) :: pbin, p
   integer, intent(in) :: n
   integer, intent(out) :: ier

   !  Local declarations
   !  ------------------

   real(kind=8), external :: binomcdf, binompmf

   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: tol=1.0e-14_8
   integer, parameter :: limit=5 ! Limit over which general inverse used
   real(kind=8) :: c, e, t, t1, t2, x
   integer :: a, b, i, m

   !--------------------------------------------------------------------

   ier = 0

   !  Test for valid input arguments

   if ( n <= 0 .or. pbin < zero .or. pbin > one    &
               .or. p < zero .or. p > one ) then
      ibinom = -1
      ier = 1
      return
   end if

   if ( pbin <= tol .or. p <= tol ) then
      ibinom = 0
   else if ( pbin >= one-tol .or. p >= one-tol ) then
      ibinom = n
   else if ( n <= limit ) then
      t = zero
      do i = 0, n-1
         t = t + binompmf( i, n, p, ier )
         if ( ier /= 0 ) return
         if ( t >= pbin ) then
            ibinom = i
            return
         end if
      end do
      ibinom = n
   else
      t = binompmf( 0, n, p, ier )   ! Zero ?
      if ( ier /= 0 ) return
      if ( t >= pbin ) then
         ibinom = 0
         return
      end if
      t = binompmf( n, n, p, ier )   ! N ?
      if ( ier /= 0 ) return
      if ( t > one-pbin ) then
         ibinom = n
         return
      end if
         ! Use normal approximation as starting point and loop
      c  = pbin
      if ( c > half ) c = one - c
      t2 = log( one/(c*c) )
      t1 = sqrt(t2)
      x  = t1 - (2.515517_8+0.802853_8*t1+0.010328_8*t2)/             &
                (1.0_8+1.432788_8*t1+0.189269_8*t2+0.001308_8*t1*t2)
      if ( pbin < half ) x = -x
      e = n*p
      c = sqrt(e*(one-p))
      m = nint(e+x*c)
      m = min( m, n-1 )
      m = max( m, 1 )
      a = 0
      b = n
      do                   ! Halving loop
         t  = binomcdf( m, n, p, ier )
         if ( ier /= 0 ) return
         if ( abs(t-pbin) <= tol ) then
            ibinom = m
            return
         else if ( t < pbin ) then
            a = m
         else
            t = t - binompmf( m, n, p, ier )
            if ( ier /= 0 ) return
            if ( t < pbin ) then
               ibinom = m
               return
            end if
            b = m
         end if
         m = (a+b)/2
      end do
   end if

end function ibinom
