subroutine eci1pdd( epsilon, type, x, n, alpha, plow, pupp, ier )
!-----------------------------------------------------------------------
!     Returns the confidence limits PLOW and PUPP, given
!     EPSILON (precision required), X (value of the variate),
!     N (sample size), ALPHA (1-alpha confidence level), and
!     TYPE (distribution name among
!                     bin : binomial
!                     bng : negative binomial
!                     poi : poisson
!                     hyp : hypergeometric
!           N.B. Hereafter only bin and bng are implemented).
!     IER is a return code (0 if OK).
!-----------------------------------------------------------------------
   implicit none
   real(kind=8), intent(in) :: epsilon, alpha
   integer, intent(in) :: x, n
   character(len=*), intent(in) :: type
   real(kind=8), intent(out) :: plow, pupp
   integer, intent(out) :: ier

   real(kind=8), external :: binomupp, nbinomlow, nbinomupp

   real(kind=8), parameter :: zero=0.0_8, one=1.0_8
   integer :: iok
   !--------------------------------------------------------------------

   plow = -one
   pupp = -one
   !  Test for valid input arguments
   if ( n <= 0 .or. x < 0 .or. alpha <= zero .or. alpha >= one ) then
      ier = 1
      return
   end if
   if ( type(:3) == "bin" ) then
      if ( x > n ) then
         ier = 1
         return
      end if
      pupp = binomupp( epsilon, alpha, x, n, ier )
      plow = one - binomupp( epsilon, alpha, n-x, n, iok )
      ier = 10*ier + iok
   else if ( type(:3) == "bng" ) then
      pupp = nbinomupp( epsilon, alpha, x, n, ier )
      plow = nbinomlow( epsilon, alpha, x, n, iok )
      ier = 10*ier + iok
!! else if ( type(:3) == "poi" ) then
!! else if ( type(:3) == "hyp" ) then
   end if
end subroutine eci1pdd

real(kind=8) function binomupp( epsilon, alpha, x, n, ier )
! Returns upper limit for binomial distribution
   implicit none
   real(kind=8), external :: binompmf, ibeta, upperlimit
   integer, external :: ibinom
   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: explower=-706.893_8
   real(kind=8) :: epsilon, alpha, thetamin, rx, u1, u2, w, zp
   integer :: x, n, ier, xp

   ier = 0
   if ( x == n ) then
      binomupp = one
      return
   end if
   rx = x
      ! U_alpha
   u1 = ibeta( one-alpha, x+one, n-rx, ier )
   if ( x == n-1 ) then   ! Particular case x=n-1
      binomupp = u1
      return
   end if
      ! U_alpha/2
   u2 = ibeta( one-alpha*half, x+one, n-rx, ier )
      ! x+
   xp = ibinom( one-alpha*half, n, u2, ier ) + 1
   if ( xp == n+1 ) then   ! Particular case x+=n+1
      w = binompmf( n, n, u1, ier )
      if ( w > alpha ) then
         binomupp = u1
         return
      end if
   end if
      ! Check theta
   thetamin = one
   if ( xp <= n ) then
      zp = xp
      w = dlgama(zp) + dlgama(n+1-zp) - dlgama(rx+one) - dlgama(n-rx)
      w = w / (zp-(rx+one))
      if ( w >= explower ) then
         w = exp(w)
         thetamin = w/(one+w)
      else
         thetamin = zero
      end if
   else
      thetamin = u2
   end if
   binomupp = upperlimit( alpha, epsilon, u1, u2, thetamin, x, xp, n, ier, "binu" )
end function binomupp

real(kind=8) function nbinomupp( epsilon, alpha, x, s, ier )
! Returns upper limit for negative binomial distribution
   implicit none
   real(kind=8), external :: nbinompmf, ibeta, upperlimit
   integer, external :: inbinom
   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: explower=-706.893_8
   real(kind=8) :: epsilon, alpha, rs, thetamin, rx, u1, u2, w
   integer :: x, s, ier, xp

   ier = 0
   if ( x == 0 ) then
      nbinomupp = one
      return
   end if
   rs = s
   rx = x
      ! U_alpha
   u1 = ibeta( one-alpha, rs, rx, ier )
      ! x+
   if ( x == 1 ) then   ! Particular case x=1
      nbinomupp = u1
      return
   end if
      ! U_alpha/2
   u2 = ibeta( one-alpha*half, rs, rx, ier )
   xp = -inbinom( alpha*half, s, u2, ier ) + 1
   if ( xp == 1 ) then   ! Particular case x+=1
      w = nbinompmf( 0, s, u1, ier )
      if ( w > alpha ) then
         nbinomupp = u1
         return
      end if
   end if
      ! Check theta
   if ( xp < 1 ) then
      w = dlgama(one-xp) + dlgama(rs+rx) - dlgama(rx) - dlgama(s-xp+one)
      w = w / (1-xp-rx)
      if ( w >= explower ) then
         thetamin = one - exp(w)
      else
         thetamin = one
      end if
   else
      thetamin = u1
   end if
   nbinomupp = upperlimit( alpha, epsilon, u1, u2, thetamin, -x, xp, s, ier, "bngu" )
end function nbinomupp

real(kind=8) function nbinomlow( epsilon, alpha, x, s, ier )
! Returns lower limit for negative binomial distribution
   implicit none
   real(kind=8), external :: ibeta, upperlimit
   integer, external :: inbinom
   real(kind=8), parameter :: zero=0.0_8, half=0.5_8, one=1.0_8
   real(kind=8), parameter :: explower=-706.893_8
   real(kind=8) :: epsilon, alpha, l1, l2, rs, thetamin, w, zp
   integer :: x, s, ier, xp

   rs = s
      ! L_alpha/2
   l2 = ibeta( one-alpha*half, x+one, rs, ier )
      ! L_alpha
   l1 = ibeta( one-alpha, x+one, rs, ier )
      ! x+
   xp = inbinom( one-alpha*half, s, one-l2, ier ) + 1
      ! Check theta
   zp = xp
   w = dlgama(zp) + dlgama(s+x+one) - dlgama(x+one) - dlgama(s+zp)
   w = w / (zp-(x+1))
   if ( w >= explower ) then
      thetamin = exp(w)
   else
      thetamin = zero
   end if
   w = upperlimit( alpha, epsilon, l1, l2, thetamin, x, xp, s, ier, "bngl" )
   nbinomlow = one - w
end function nbinomlow
